from abc import abstractmethod
from dataclasses import dataclass
from typing import Protocol

from elkar.a2a_types import *
from elkar.common import PaginatedResponse


@dataclass
class StoredTask:
    id: str
    caller_id: str | None
    session_id: str | None
    task: Task
    push_notification: PushNotificationConfig | None
    created_at: datetime
    updated_at: datetime


@dataclass
class UpdateTaskParams:
    status: TaskStatus | None = None
    artifacts_updates: list[Artifact] | None = None
    new_messages: list[Message] | None = None
    metadata: dict[str, Any] | None = None
    push_notification: PushNotificationConfig | None = None
    caller_id: str | None = None


class ListTasksOrder(str, Enum):
    CREATED_AT = "created_at"
    UPDATED_AT = "updated_at"


@dataclass
class ListTasksParams:
    caller_id: str | None = None
    state_in: list[TaskState] | None = None
    order_by: ListTasksOrder = ListTasksOrder.CREATED_AT
    page: int | None = 1
    page_size: int | None = 100


class TaskManagerStore(Protocol):

    @abstractmethod
    async def upsert_task(
        self, params: TaskSendParams, caller_id: str | None = None
    ) -> StoredTask:
        """
        Upsert a task. Should create a new task if it doesn't exist, or update an existing task.
        Update rules are as follows:
        - If the task exists:
            - update the status
            - append message to history
            - update metadata
        - If the task does not exist, create a new task with the given params.
        - If the task exists and the caller_id is different, raise an error.
        UpsertTaskParams:
            id: str
            caller_id: str | None
            status: TaskState
            history: list[Message]
            artifacts: list[Artifact]
            metadata: dict[str, Any]
        """
        ...

    @abstractmethod
    async def get_task(
        self,
        task_id: str,
        history_length: int | None = None,
        caller_id: str | None = None,
    ) -> StoredTask:
        """
        Get the task with the following rules:
        - If history_length is provided, return the last N messages
        - If caller_id is provided, return the task only if the caller_id matches
        """
        ...

    @abstractmethod
    async def update_task(
        self,
        task_id: str,
        params: UpdateTaskParams,
    ) -> StoredTask:
        """
        Update the task with the following rules:
        - If status is provided, update the status
        - If artifacts is provided, update the artifacts (i.e. if an artifact with the same index already exists, it will be updated: the only authorized update is to append to the parts list, if the artifact does not exist, it will be created)
        - If caller_id is provided, update the caller_id
        """
        ...

    @abstractmethod
    async def list_tasks(
        self, params: ListTasksParams
    ) -> PaginatedResponse[StoredTask]:
        """
        List tasks with the following rules:
        - If caller_id is provided, return the tasks only if the caller_id matches
        """
        ...



import asyncio
import logging
from typing import AsyncIterable
from elkar.a2a_types import *
import colorlog
from elkar.task_queue.in_memory import InMemoryTaskEventQueue
from elkar.server.server import A2AServer
from elkar.store.base import TaskManagerStore
from elkar.task_manager.task_manager_base import RequestContext
from elkar.task_manager.task_manager_with_store import (
    TaskManagerWithStore,
    TaskSendOutput,
)
from elkar.store.in_memory import InMemoryClientTaskManagerBackend

store = InMemoryClientTaskManagerBackend()

handler = colorlog.StreamHandler()
handler.setFormatter(
    colorlog.ColoredFormatter(
        "%(log_color)s%(levelname)s%(reset)s - %(asctime)s - %(name)s - %(funcName)s - %(pathname)s:%(lineno)d - %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
        log_colors={
            "DEBUG": "cyan",
            "INFO": "green",
            "WARNING": "yellow",
            "ERROR": "red",
            "CRITICAL": "red,bg_white",
        },
    )
)

logger = logging.getLogger()
logger.addHandler(handler)
logger.setLevel(logging.DEBUG)

agent_card = AgentCard(
    name="Test Agent",
    description="Test Agent Description",
    url="https://example.com",
    version="1.0.0",
    skills=[],
    capabilities=AgentCapabilities(
        streaming=True,
        pushNotifications=True,
        stateTransitionHistory=True,
    ),
)

queue = InMemoryTaskEventQueue()


async def send_task(
    task: Task, request_context: RequestContext | None, store: TaskManagerStore | None
) -> TaskSendOutput:

    return TaskSendOutput(
        status=TaskStatus(
            state=TaskState.COMPLETED,
            message=None,
            timestamp=datetime.now(),
        ),
        metadata=task.metadata,
        new_artifacts=[
            Artifact(
                parts=[
                    TextPart(
                        text="Hello, Artifact!",
                    )
                ],
                index=0,
                lastChunk=True,
            )
        ],
        new_history_messages=[
            Message(
                role="agent",
                parts=[
                    TextPart(text="Hello, Message!"),
                ],
            )
        ],
    )


async def send_task_streaming(
    task: Task, request_context: RequestContext | None, store: TaskManagerStore | None
) -> AsyncIterable[SendTaskStreamingResponse]:

    yield SendTaskStreamingResponse(
        result=TaskStatusUpdateEvent(
            id=task.id,
            status=TaskStatus(
                state=TaskState.WORKING,
                message=None,
            ),
            metadata=task.metadata,
            final=False,
        ),
    )

    await asyncio.sleep(1)

    yield SendTaskStreamingResponse(
        result=TaskArtifactUpdateEvent(
            id=task.id,
            artifact=Artifact(
                index=0,
                parts=[
                    TextPart(text="Hello, world!"),
                    TextPart(text="Hello, world!"),
                ],
            ),
            metadata=task.metadata,
        ),
    )

    await asyncio.sleep(1)

    yield SendTaskStreamingResponse(
        result=TaskArtifactUpdateEvent(
            id=task.id,
            artifact=Artifact(
                index=0,
                parts=[
                    TextPart(text="Hello, world!"),
                    TextPart(text="Hello, world!"),
                ],
            ),
            metadata={},
        ),
    )

    await asyncio.sleep(1)

    yield SendTaskStreamingResponse(
        result=TaskArtifactUpdateEvent(
            id=task.id,
            artifact=Artifact(
                name="test",
                description="test",
                index=1,
                parts=[
                    TextPart(text="Hello, world!"),
                    TextPart(text="Hello, world!"),
                ],
            ),
            metadata=task.metadata,
        ),
    )

    await asyncio.sleep(1)

    yield SendTaskStreamingResponse(
        result=TaskStatusUpdateEvent(
            id=task.id,
            status=TaskStatus(state=TaskState.COMPLETED, message=None),
            metadata=task.metadata,
            final=True,
        ),
    )


task_manager = TaskManagerWithStore(
    store,
    agent_card,
    queue,
    send_task_handler=send_task,
    send_task_streaming_handler=send_task_streaming,
)


# Create the server instance but don't start it
server = A2AServer(task_manager, host="0.0.0.0", port=5001, endpoint="/")

# Expose the Starlette application instance
app = server.app



from abc import abstractmethod
from dataclasses import dataclass
from typing import Any, AsyncIterable, Protocol
from elkar.a2a_types import (
    AgentCard,
    CancelTaskRequest,
    SendTaskRequest,
    SendTaskResponse,
    GetTaskRequest,
    GetTaskResponse,
    CancelTaskResponse,
    SendTaskStreamingRequest,
    SendTaskStreamingResponse,
    SetTaskPushNotificationRequest,
    SetTaskPushNotificationResponse,
    GetTaskPushNotificationRequest,
    GetTaskPushNotificationResponse,
    TaskResubscriptionRequest,
    JSONRPCResponse,
)
from elkar.common import ListTasksRequest, PaginatedResponse, TaskResponse


@dataclass
class RequestContext:
    caller_id: str | None
    metadata: dict[str, Any]


class TaskManager(Protocol):
    @abstractmethod
    async def get_agent_card(self) -> AgentCard: ...

    @abstractmethod
    async def send_task(
        self, request: SendTaskRequest, request_context: RequestContext | None = None
    ) -> SendTaskResponse: ...

    @abstractmethod
    async def get_task(
        self, request: GetTaskRequest, request_context: RequestContext | None = None
    ) -> GetTaskResponse: ...

    @abstractmethod
    async def cancel_task(
        self, request: CancelTaskRequest, request_context: RequestContext | None = None
    ) -> CancelTaskResponse: ...

    @abstractmethod
    async def send_task_streaming(
        self,
        request: SendTaskStreamingRequest,
        request_context: RequestContext | None = None,
    ) -> AsyncIterable[SendTaskStreamingResponse] | JSONRPCResponse: ...

    @abstractmethod
    async def set_task_push_notification(
        self,
        request: SetTaskPushNotificationRequest,
        request_context: RequestContext | None = None,
    ) -> SetTaskPushNotificationResponse: ...

    @abstractmethod
    async def get_task_push_notification(
        self,
        request: GetTaskPushNotificationRequest,
        request_context: RequestContext | None = None,
    ) -> GetTaskPushNotificationResponse: ...

    @abstractmethod
    async def resubscribe_to_task(
        self,
        request: TaskResubscriptionRequest,
        request_context: RequestContext | None = None,
    ) -> AsyncIterable[SendTaskStreamingResponse] | JSONRPCResponse: ...

    async def list_tasks(
        self, request: ListTasksRequest
    ) -> PaginatedResponse[TaskResponse]:
        raise NotImplementedError("list_tasks is not implemented")
